/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.v2.xml;

import javax.annotation.concurrent.Immutable;
import net.java.truelicense.core.*;
import net.java.truelicense.obfuscate.Obfuscate;
import net.java.truelicense.v2.core.BasicV2LicenseManagementContext;
import net.java.truelicense.xml.*;

/**
 * The root context for the management of V2/XML format license keys.
 * Note that there is no interoperability between different format license keys.
 * <p>
 * Use this context to create a {@link LicenseVendorContext} or a
 * {@link LicenseConsumerContext}.
 * Here's an example for verifying the installed license key in a consumer
 * application:
 * <pre><code>
 * LicenseManagementContext mc = new V2XmlLicenseManagementContext("MyApp 1");
 * LicenseConsumerContext cc = mc.consumer();
 * LicenseConsumerManager cm = cc.manager(...);
 * cm.verify();
 * </code></pre>
 * <p>
 * Or in a fluent API programming style:
 * <pre><code>
 * new V2XmlLicenseManagementContext("MyApp 1").consumer().manager()...verify();
 * </code></pre>
 * <p>
 * Do not copy-paste this code!
 * Instead, use the TrueLicense Maven Archetype to generate a fully populated
 * template project for you.
 * <p>
 * Where required, this class should get subclassed to customize license
 * {@linkplain #license creation},
 * {@linkplain #initialization initialization}
 * or {@linkplain #validation validation},
 * {@linkplain #codec encoding} or to implement an authoritative
 * {@linkplain #clock clock} or to provide an alternative
 * {@linkplain #classLoader class loader} for loading key stores.
 * <p>
 * Note that this class is immutable.
 * Unless stated otherwise, all no-argument methods need to return consistent
 * objects so that caching them is not required.
 * A returned object is considered to be consistent if it compares
 * {@linkplain Object#equals(Object) equal} or at least behaves identical to
 * any previously returned object.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public class V2XmlLicenseManagementContext
extends BasicV2LicenseManagementContext {

    private final LicenseXmlCodec codec = new LicenseXmlCodec();

    /**
     * Constructs a V2/XML license management context.
     * The provided string should get computed on demand from an obfuscated
     * form, e.g. by annotating a constant string value with the
     * {@literal @}{@link Obfuscate} annotation and processing it with the
     * TrueLicense Maven Plugin.
     *
     * @param subject the licensing subject, i.e. a product name with an
     *        optional version range, e.g. {@code MyApp 1}.
     */
    public V2XmlLicenseManagementContext(String subject) { super(subject); }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V2XmlLicenseManagementContext}
     * returns a new {@link XmlRepository}.
     */
    @Override public final XmlRepository repository() {
        return new XmlRepository(codec(License.class)); // FIXME!
    };

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V2XmlLicenseManagementContext}
     * returns a {@link LicenseXmlCodec}.
     */
    @Override public LicenseXmlCodec codec(Class<?> expected) { return codec; }
}
